/**********************************************************************
*
*	DXL:		mod_crsync.dxl
*	Instance:		1
*	Description:	Synergy/DOORS Integration
*			Synchronize the "Change Request" and "CR Status" attributes
*			in all the requirement objects in current module from ChangeSynergy
*	%created_by:	usmave %
*	%date_created:	Tuesday, October 27, 2009 8:42:29 AM %
*
*   Copyright (c) 2001 Telelogic AB
**********************************************************************/

#include <addins/synergy/utils/lib.inc>
#include <addins/synergy/utils/csint_util.inc>
#include <addins/synergy/utils/req_object_update.inc>
#include <addins/synergy/Utils/ecpsutils.inc>
#include <addins/synergy/Utils/ecpsModuleConfiguration.inc>

// =====================================================================================
bool mSectionPushToCs(Module mdl, int progress, OleAutoObj obj_csApiLib, obj_csUser, string queryString)
{
	bool continueSyncProcess = true
	Module m = mdl
	string msg1 = getCsintString("csString_ModuleSyncIR_ProgressMsg_UpdateCS", CSNAME)
	string msg2 = getCsintString("csString_ModuleSyncIR_ProgressMsg_QueryReqs")
	progressMessage(msg1 "\n\t" msg2)
	csintDebug_("Query CS for associated req objects with query string = " queryString)
	OleAutoObj obj_csQueryData = do_csApiLib_QueryStringData(obj_csApiLib, obj_csUser, "", queryString, "name|cvid|doors_modify_time")
	csintDebug_("Query return.")
	if (!null obj_csQueryData)
	{
		int numResults = get_int_from_OleAutoObj(obj_csQueryData, "DataSize")
		csintDebug_("The number of associated req objects return by query is " numResults ".")
		int j
		for(j=0;j<numResults;j++)
		{
			progressStep(progress)
			if (progressCancelled)
			{
				csintDebug_("User cancel the sync process!!")
				continueSyncProcess = false
				break
			}

			OleAutoObj obj_csObjectVector = get_indexed_OleAutoObj_from_OleAutoObj(obj_csQueryData, "DataObject", j)
			if (!null obj_csObjectVector)
			{
				string cvid
				int modtime
				string reqName
				int i
				for (i=0; i<3; i++)
				{
					OleAutoObj obj_csObjectData = get_indexed_OleAutoObj_from_OleAutoObj(obj_csObjectVector, "DataObject", i)
					string oName = get_string_from_OleAutoObj(obj_csObjectData, "Name")
					
					if (oName == "cvid")
					{
						cvid = get_string_from_OleAutoObj(obj_csObjectData, "Value")
					}
					else if (oName == "doors_modify_time")
					{
						modtime = intOf(get_string_from_OleAutoObj(obj_csObjectData, "Value"))
					}
					else if (oName == "name")
					{
						reqName = get_string_from_OleAutoObj(obj_csObjectData, "Value")
					}
					else
					{
						continue
					}
				}

				string oID = getLastElement(reqName, '_')
				csintDebug_("Data[" j "] cvid=" cvid " doors_modify_time=" modtime " objAbsNo=" oID "")
				string msg3 = getCsintString("csString_ModuleSyncIR_ProgressMsg_ForReq", oID)
				progressMessage(msg1 "\n\t" msg3)
				Object obj = findThisObject(mdl, oID)
				push_Obj_Data_to_CS(obj_csApiLib, obj_csUser, obj, cvid, modtime, m)		
			}
			else
			{
				csintDebug_("Data[" j "] is null")
			}
		}
		cleanupQueryData(obj_csQueryData)
	}

	return continueSyncProcess
}

// =====================================================================================
bool mSectionUpdateDOORS(Module mdl, int progress, OleAutoObj obj_csApiLib, obj_csUser, string secondQueryString, queryString, attrList)
{
	bool continueSyncProcess = true
	int iNumECRs = 0
	int iNumReqs = 0

	string msg1 = getCsintString("csString_ModuleSyncIR_ProgressMsg_UpdateIRs", ECR)
	string msg2 = getCsintString("csString_ModuleSyncIR_ProgressMsg_QueryIRs")
	progressMessage(msg1 "\n\t" msg2 )
	csintDebug_("Query CS for associated IRs with query string = " secondQueryString)
	OleAutoObj csQueryDataCR = do_csApiLib_QueryStringData(obj_csApiLib, obj_csUser, "", secondQueryString, attrList)
	csintDebug_("Query return.")
	if (!null csQueryDataCR)
	{
		iNumECRs = get_int_from_OleAutoObj(csQueryDataCR, "DataSize")
		csintDebug_("The number of associated ECRs return by query is " iNumECRs ".")
	}
	string msg3 = getCsintString("csString_ModuleSyncIR_ProgressMsg_QueryReqs")
	progressMessage(msg1 "\n\t" msg3)
	csintDebug_("Query CS for associated req objects using doors_data_report_req_cr report with query string = " queryString)
	OleAutoObj csQueryDataReq2CR = do_csApiLib_QueryStringData(obj_csApiLib, obj_csUser, "doors_data_report_req_cr", queryString,"name|cvid")
	csintDebug_("Query return.")
	if (!null csQueryDataReq2CR)
	{
		iNumReqs = get_int_from_OleAutoObj(csQueryDataReq2CR, "DataSize")
		csintDebug_("The number of associated req objects return by query is " iNumReqs ".")
	}

	if ((iNumECRs>0)&&(iNumReqs>0)) then
	{
		Skip dataSkip = createString
		indexCRInQueryData(csQueryDataCR, dataSkip)
		Buffer cacheBuffer = create

		int iReq
		for (iReq=0; iReq<iNumReqs; iReq++)
		{
			progressStep(progress)
			if (progressCancelled)
			{
				csintDebug_("User cancel the sync process!!")
				continueSyncProcess = false
				break
			}

			OleAutoObj reqObj = get_indexed_OleAutoObj_from_OleAutoObj(csQueryDataReq2CR, "DataObject", iReq)
			if (!null reqObj)
			{
				string errMsg = ""
				string doorsID = getRecordField(reqObj, "name", errMsg)
				string objID = getLastElement(doorsID, '_')
				csintDebug_("Data[" iReq "] for object " objID)
				string moduleSyncIR_ProgressMsg_ForReq = getCsintString("csString_ModuleSyncIR_ProgressMsg_ForReq", objID)
				progressMessage(msg1 "\n\t" moduleSyncIR_ProgressMsg_ForReq)
				Object obj = findThisObject(mdl, objID)
				if (!null obj)
				{
					update_IR_StatusCache(mdl, obj_csApiLib, obj_csUser, obj, dataSkip, reqObj, cacheBuffer)
				}
				else
				{
					csintDebug_("\tObject(" objID ") can not be found in DOORS! Skip update this object in DOORS.")
				}
			}
			else
			{
				csintDebug_("Data[" iReq "] is null")
			}
		}
		delete dataSkip
		delete cacheBuffer
	}

	if (!null csQueryDataCR) then cleanupQueryData(csQueryDataCR)
	if (!null csQueryDataReq2CR) then cleanupQueryData(csQueryDataReq2CR)

	return continueSyncProcess
}

// =====================================================================================
void module_ir_sync(string viewName)
{
	csintDebug_("module_ir_sync(" viewName ") start...")
	int startTime = intOf today()

	Object cur_obj = current Object
	Module mdl = current Module
	if (null mdl)
	{
		string csint_ErrMsg_NullCurrentModule = getCsintString("csString_Csint_ErrMsg_NullCurrentModule")
		abortMsg(csint_ErrMsg_NullCurrentModule)
	}
	if (!canRead mdl)
	{
		string csint_ErrMsg_NoReadAccessToModule = getCsintString("csString_Csint_ErrMsg_NoReadAccessToModule")
		abortMsg(csint_ErrMsg_NoReadAccessToModule)
	}
	//if (!canWrite mdl)
	//	abortMsg(csString_Csint_ErrMsg_NoWriteAccessToModule)

	Item modItem = item fullName mdl
	string mid = uniqueID modItem

	check_module_initialization(mdl)

	string string_Progress = getCsintString("csString_String_Progress")
	string moduleSyncIR_ProgressTitle = getCsintString("csString_ModuleSyncIR_ProgressTitle")
	string moduleSyncIR_ProgressMsg_Init = getCsintString("csString_ModuleSyncIR_ProgressMsg_Init")
	DB myBox = create(mdl, string_Progress)
	realize myBox
	progressStart(myBox, moduleSyncIR_ProgressTitle, moduleSyncIR_ProgressMsg_Init, 45)
	progressMessage(moduleSyncIR_ProgressMsg_Init)
	int progress = 0

	string attrList = ECPS_DEFAULT_CACHE_QUERY_ATTRS
	if IR_USE_STATUS_VIEW then getAttrListFromViewConfig(IR_STATUS_VIEW, attrList)
	if IR_USE_ATTRS_VIEW then getAttrListFromViewConfig(IR_ATTRS_VIEW, attrList)

	OleAutoObj obj_csUser = null
	OleAutoObj obj_csApiLib = get_csApiLib(obj_csUser)
	Buffer b = create
	int xNumber
	bool continueSyncProcess = true

	progress++
	progressStep(progress)
	if (progressCancelled) then
	{
		csintDebug_("User cancel the sync process!!")
		continueSyncProcess = false
	}

	if continueSyncProcess then
	{
		csintDebug_("====================================================")
		csintDebug_("Phase.1 Update requirement objects in Change first")
		csintDebug_("====================================================")
		for(xNumber=0; xNumber<10; xNumber++)
		{
			progress++
			progressStep(progress)
			if (progressCancelled) then
			{
				csintDebug_("User cancel the sync process!!")
				continueSyncProcess = false
				break
			}

			string moduleSyncIR_ProgressMsg_UpdateCS = getCsintString("csString_ModuleSyncIR_ProgressMsg_UpdateCS", CSNAME)
			progressMessage(moduleSyncIR_ProgressMsg_UpdateCS)
			csintDebug_("push to cs -----------------------------> " xNumber "...")
			string req_name = "req" mid "_" xNumber "*"

			setempty b
			b += "name match '"
			b += req_name
			b += "'"
			string queryString = stringOf b

			continueSyncProcess = mSectionPushToCs(mdl, progress, obj_csApiLib, obj_csUser, queryString)
			if !continueSyncProcess then break
		}
		csintDebug_("end of push to cs.")

		csintDebug_("====================================================")
		csintDebug_("Phase.2 Update IR info in DOORS")
		csintDebug_("====================================================")
		for(xNumber=0; xNumber<10; xNumber++)
		{
			progress += 3
			progressStep(progress)
			if (progressCancelled) then
			{
				csintDebug_("User cancel the sync process!!")
				continueSyncProcess = false
				break
			}

			string moduleSyncIR_ProgressMsg_UpdateIRs = getCsintString("csString_ModuleSyncIR_ProgressMsg_UpdateIRs", ECR)
			progressMessage(moduleSyncIR_ProgressMsg_UpdateIRs)
			csintDebug_("update doors -----------------------------> " xNumber "...")
			string req_name = "req" mid "_" xNumber "*"

			setempty b
			b += "name match '"
			b += req_name
			b += "'"
			string queryString = stringOf b

			setempty b
			b += "has_associated_req(name match '"
			b += req_name
			b += "')"
			string secondQueryString = stringOf b

			continueSyncProcess = mSectionUpdateDOORS(mdl, progress, obj_csApiLib, obj_csUser, secondQueryString, queryString, attrList)
			if !continueSyncProcess then break
		}
		csintDebug_("end of update doors.")
	}

	delete b

	progressStop()
	hide myBox
	destroy myBox

	refresh mdl
	// only load the view if it is a status view
	if (viewName == IR_STATUS_VIEW) then
	{
		if currentView(mdl) != viewName then load(mdl, view(viewName))
	}

	// remember the current object focus
	if (!null cur_obj) then 
	{
		scroll to cur_obj
		current = cur_obj
	}

	refresh mdl
	int endTime = intOf today()
	csintDebug_("module_ir_sync() end and took " (endTime-startTime) " seconds")
}
