#include <addins/synergy/Utils/lib.inc>
#include <addins/synergy/Utils/csint_util.inc>
#include <addins/synergy/Utils/ecpsutils.inc>
#include <addins/synergy/ecps2csApi/ecps_csapi.inc>
#include <addins/synergy/Utils/ecpsModuleConfiguration.inc>

// =======================================================================
// Global data structures, DBs, and DBEs
// =======================================================================
string csint_status_view_target = ""
int csint_status_view_type = CSINT_STATUS_VIEW_TYPE_DXL_ATTR

// Data structure for holding configurations data
//
string queryAttrs = null
int activeConfig = -1
Skip configNameSkip = create
Skip configAppliedSkip = create
Skip configColumnLabelSkip = create
Skip configColumnAttrsSkip = create

string customQueryAttrs = null
bool customIncludeApplied = false
Skip customMappings = createString
DBE customIncludeAppliedToggle = null

// Three dialogs for config list, config details, and column details
//
DB viewConfigDB = null
DBE configListview = null

DB configDetailsDB = null
DBE configNameField = null
DBE configIncludeAppliedToggle = null
DBE columnListview = null

DB columnDetailsDB = null
DBE columnLabelField = null
DBE attrListview = null
DBE candidateListview = null

// Flags used by parent dialog to check which button ( Cancel or OK ) is used to close its child dialog
//
bool configDetailsOK = false
bool columnDetailsOK = false

// Temp working data structure
//
string workingConfigName = null
bool workingIncludeApplied = false
Skip workingLabelSkip = null
Skip workingAttrsSkip = null

string workingColumnLabel = null
string workingColumnAttrs = null

string convertCsAttrListToLabelList(string attrListString, Skip candidateAttrs)
{
	Buffer b = create

	string cAttrs = attrListString
	string aAttribute = getNextElement(cAttrs, ',')
	string attrLabel = null
	while (!null aAttribute)
	{
		if find(candidateAttrs, aAttribute, attrLabel) then 
		{
			if (length(b) > 0) then
			{
				b += ", "
			}
			b += attrLabel
		}
		else
		{
			// maybe need to notice user the attribute no longer exist in cs...
		}
		aAttribute = getNextElement(cAttrs, ' ')
		aAttribute = getNextElement(cAttrs, ',')
	}
	string result = stringOf b
	delete b
	return result
}

// =======================================================================
// Column Details Dialog
// =======================================================================

void displayColumnDetails(DBE focusDBE)
{
	set(columnLabelField, workingColumnLabel)

	OleAutoObj obj_csUser = null
	OleAutoObj obj_csApiLib = get_csApiLib(obj_csUser)
	Skip candidateAttrs = createString
	getReportableCRAttributeList(obj_csApiLib, obj_csUser, candidateAttrs)
	//getCurrentModuleAttrs(candidateAttrs)
	if csint_status_view_target == IR_STATUS_VIEW then put(candidateAttrs, CSINT_DOORS_OBJECT_ATTR_CM_BASELINE, "_CM_BASELINE")

	string cAttrs = workingColumnAttrs
	string aAttribute = getNextElement(cAttrs, ',')
	string attrLabel = null
	int j = 0
	while (!null aAttribute)
	{
		if find(candidateAttrs, aAttribute, attrLabel) then 
		{
			insert(attrListview, j, attrLabel, iconNone)
			set(attrListview, j, 1, aAttribute)
			j += 1
			delete(candidateAttrs, aAttribute)
		}
		else
		{
			// maybe need to notice user the attribute no longer exist in cs...
		}
		aAttribute = getNextElement(cAttrs, ' ')
		aAttribute = getNextElement(cAttrs, ',')
	}

	j = 0
	for attrLabel in candidateAttrs do
	{
		aAttribute = (string key candidateAttrs)
		insert(candidateListview, j, attrLabel, iconNone)
		set(candidateListview, j, 1, aAttribute)
		j += 1
	}

	delete candidateAttrs

	setFocus(focusDBE)
}

void clearSelection(DBE aListview)
{
	int i = 0
	for (i=0; i<noElems(aListview); i++)
	{
		set(aListview, i, false)
	}
}

void moveAttributes(DBE fromDBE, toDBE)
{
	Skip selectedAttrIndexes = create
	Skip AttrNeedtobeAdd = create
	Skip AttrNameNeedtobeAdd = create
	string AttrSelected = null
	string AttrNameSelected = null
	int selectedAttrIndex
	for selectedAttrIndex in fromDBE do
	{
		put(selectedAttrIndexes, -selectedAttrIndex, selectedAttrIndex)
	}

	for selectedAttrIndex in selectedAttrIndexes do
	{
		AttrSelected = getColumnValue(fromDBE, selectedAttrIndex, 0)
		put(AttrNeedtobeAdd, selectedAttrIndex, AttrSelected)
		AttrNameSelected = getColumnValue(fromDBE, selectedAttrIndex, 1)
		put(AttrNameNeedtobeAdd, selectedAttrIndex, AttrNameSelected)
		delete(fromDBE, selectedAttrIndex)
	}

	clearSelection(toDBE)
	for AttrSelected in AttrNeedtobeAdd do
	{
		selectedAttrIndex = (int key AttrNeedtobeAdd)
		find(AttrNameNeedtobeAdd, selectedAttrIndex, AttrNameSelected)
		int insertPosition = noElems(toDBE)
		insert(toDBE, insertPosition, AttrSelected, iconNone)
		set(toDBE, insertPosition, 1, AttrNameSelected)
		set(toDBE, insertPosition, true)
	}

	delete selectedAttrIndexes
	delete AttrNeedtobeAdd
	delete AttrNameNeedtobeAdd
}


void rowUp(DBE targetDBE, bool doColumn2)
{
	Skip selectedAttrIndexes = create
	string AttrSelected = null
	string AttrAbove = null
	string column1Selected = null
	string column1Above = null
	int selectedAttrIndex
	for selectedAttrIndex in targetDBE do
	{
		put(selectedAttrIndexes, selectedAttrIndex, selectedAttrIndex)
	}

	for selectedAttrIndex in selectedAttrIndexes do
	{
		if ((selectedAttrIndex > 0) && !selected(targetDBE, selectedAttrIndex-1)) then
		{
			AttrSelected = getColumnValue(targetDBE, selectedAttrIndex, 0)
			AttrAbove = getColumnValue(targetDBE, selectedAttrIndex-1, 0)
			set(targetDBE, selectedAttrIndex, 0, AttrAbove)
			set(targetDBE, selectedAttrIndex-1, 0, AttrSelected)
			set(targetDBE, selectedAttrIndex, false)
			set(targetDBE, selectedAttrIndex-1, true)

			column1Selected = getColumnValue(targetDBE, selectedAttrIndex, 1)
			column1Above = getColumnValue(targetDBE, selectedAttrIndex-1, 1)
			set(targetDBE, selectedAttrIndex, 1, column1Above)
			set(targetDBE, selectedAttrIndex-1, 1, column1Selected)

			if doColumn2 then
			{
				column1Selected = getColumnValue(targetDBE, selectedAttrIndex, 2)
				column1Above = getColumnValue(targetDBE, selectedAttrIndex-1, 2)
				set(targetDBE, selectedAttrIndex, 2, column1Above)
				set(targetDBE, selectedAttrIndex-1, 2, column1Selected)
			}
		}
	}
	delete selectedAttrIndexes
}

void rowDown(DBE targetDBE, bool doColumn2)
{
	Skip selectedAttrIndexes = create
	string AttrSelected = null
	string AttrBelow = null
	string column1Selected = null
	string column1Below = null
	int selectedAttrIndex
	for selectedAttrIndex in targetDBE do
	{
		put(selectedAttrIndexes, -selectedAttrIndex, selectedAttrIndex)
	}

	for selectedAttrIndex in selectedAttrIndexes do
	{
		if ((selectedAttrIndex < noElems(targetDBE)-1) && !selected(targetDBE, selectedAttrIndex+1)) then
		{
			AttrSelected = getColumnValue(targetDBE, selectedAttrIndex, 0)
			AttrBelow = getColumnValue(targetDBE, selectedAttrIndex+1, 0)
			set(targetDBE, selectedAttrIndex, 0, AttrBelow)
			set(targetDBE, selectedAttrIndex+1, 0, AttrSelected)
			set(targetDBE, selectedAttrIndex, false)
			set(targetDBE, selectedAttrIndex+1, true)

			column1Selected = getColumnValue(targetDBE, selectedAttrIndex, 1)
			column1Below = getColumnValue(targetDBE, selectedAttrIndex+1, 1)
			set(targetDBE, selectedAttrIndex, 1, column1Below)
			set(targetDBE, selectedAttrIndex+1, 1, column1Selected)

			if doColumn2 then
			{
				column1Selected = getColumnValue(targetDBE, selectedAttrIndex, 2)
				column1Below = getColumnValue(targetDBE, selectedAttrIndex+1, 2)
				set(targetDBE, selectedAttrIndex, 2, column1Below)
				set(targetDBE, selectedAttrIndex+1,2, column1Selected)
			}
		}
	}
	delete selectedAttrIndexes
}

void addAttribute(DBE btn)
{
	moveAttributes(candidateListview, attrListview)
}

void removeAttribute(DBE btn)
{
	moveAttributes(attrListview, candidateListview)
}

void moveAttrUp(DBE btn)
{
	rowUp(attrListview, false)
}

void moveAttrDown(DBE btn)
{
	rowDown(attrListview, false)
}

void cancelColumnDetails(DB box)
{
	columnDetailsOK = false
	release box
}

void okColumnDetails(DB box)
{
	workingColumnLabel = get(columnLabelField)
	int i
	for (i=0; i<noElems(attrListview); i++)
	{
		if i == 0 then
		{
			workingColumnAttrs = getColumnValue(attrListview, i, 1)
		}
		else
		{
			workingColumnAttrs = workingColumnAttrs ", " getColumnValue(attrListview, i, 1)
		}
	}
	columnDetailsOK = true
	release box
}

// =======================================================================

void ecps_Column_Details_Dialog()
{
	string emptyList[]

	string columnDetail_DialogTitle = getCsintString("csString_ColumnDetail_DialogTitle")
	columnDetailsDB = create(configDetailsDB, columnDetail_DialogTitle, styleStandard)
	
	string columnDetail_Field_Label = getCsintString("csString_ColumnDetail_Field_Label")
	columnLabelField = field(columnDetailsDB, columnDetail_Field_Label, "", 50)

	attrListview = listView(columnDetailsDB, listViewOptionMultiselect, 204, 5, emptyList)
	attrListview->"right"->"unattached"
	attrListview->"bottom"->"unattached"

	string columnDetail_Button_Add = getCsintString("csString_ColumnDetail_Button_Add")
	DBE addAttrBtn = button(columnDetailsDB, columnDetail_Button_Add, addAttribute)
	addAttrBtn->"top"->"aligned"->attrListview
	addAttrBtn->"left"->"spaced"->attrListview
	
	string columnDetail_Button_Remove = getCsintString("csString_ColumnDetail_Button_Remove")
	DBE removeAttrBtn = button(columnDetailsDB, columnDetail_Button_Remove, removeAttribute)
	removeAttrBtn->"top"->"spaced"->addAttrBtn
	removeAttrBtn->"left"->"spaced"->attrListview

	candidateListview = listView(columnDetailsDB, listViewOptionMultiselect, 204, 5, emptyList)
	candidateListview->"top"->"aligned"->attrListview
	candidateListview->"left"->"spaced"->removeAttrBtn
	candidateListview->"right"->"unattached"
	candidateListview->"bottom"->"unattached"
	
	string columnDetail_Button_MoveAttrUp = getCsintString("csString_ColumnDetail_Button_MoveAttrUp")
	DBE moveAttrUpBtn = button(columnDetailsDB, columnDetail_Button_MoveAttrUp, moveAttrUp)
	moveAttrUpBtn->"top"->"spaced"->attrListview
	
	string columnDetail_Button_MoveAttrDown = getCsintString("csString_ColumnDetail_Button_MoveAttrDown")
	DBE moveAttrDownBtn = button(columnDetailsDB, columnDetail_Button_MoveAttrDown, moveAttrDown)
	moveAttrDownBtn->"top"->"aligned"->moveAttrUpBtn
	moveAttrDownBtn->"left"->"spaced"->moveAttrUpBtn

	//DBE explanation = label(columnDetailsDB, "* indicates proposed value of a DOORS attribute")
	//explanation->"top"->"spaced"->moveAttrUpBtn
	//explanation->"right"->"aligned"->moveAttrUpBtn

	close(columnDetailsDB, false, cancelColumnDetails)
	string configView_Button_Cancel = getCsintString("csString_ConfigView_Button_Cancel")
	string configView_Button_OK = getCsintString("csString_ConfigView_Button_OK")
	ok(columnDetailsDB, configView_Button_Cancel, cancelColumnDetails)
	ok(columnDetailsDB, configView_Button_OK, okColumnDetails)

	realize columnDetailsDB
	
	string columnDetail_Column_IncludedAttrs = getCsintString("csString_ColumnDetail_Column_IncludedAttrs")
	string columnDetail_Column_AvailableAttrs = getCsintString("csString_ColumnDetail_Column_AvailableAttrs")
	insertColumn(attrListview,       0, columnDetail_Column_IncludedAttrs,  200, iconNone)
	insertColumn(attrListview,       1, "",                          1, iconNone)
	insertColumn(candidateListview,  0, columnDetail_Column_AvailableAttrs, 200, iconNone)
	insertColumn(candidateListview,  1, "",                          1, iconNone)

	displayColumnDetails(columnLabelField)

	block columnDetailsDB
}

// =======================================================================
// Configuration Details Dialog
// =======================================================================

void columnListview_onSelect(DBE lv, int i)
{
	// Nothing
}

void columnListview_onDeselect(DBE lv, int i)
{
	// Nothing
}

void columnListview_onActivate(DBE lv, int i)
{
	if i<0 then return

	workingColumnLabel = getColumnValue(columnListview, i, 0)
	workingColumnAttrs = getColumnValue(columnListview, i, 2)
	ecps_Column_Details_Dialog()
	if columnDetailsOK then
	{
		set(columnListview, i, 0, workingColumnLabel)
		set(columnListview, i, 2, workingColumnAttrs)
		OleAutoObj obj_csUser = null
		OleAutoObj obj_csApiLib = get_csApiLib(obj_csUser)
		Skip candidateAttrs = createString
		getReportableCRAttributeList(obj_csApiLib, obj_csUser, candidateAttrs)
		if csint_status_view_target == IR_STATUS_VIEW then put(candidateAttrs, CSINT_DOORS_OBJECT_ATTR_CM_BASELINE, "_CM_BASELINE")
		string attrLabels = convertCsAttrListToLabelList(workingColumnAttrs, candidateAttrs)
		delete candidateAttrs
		set(columnListview, i, 1, attrLabels)
	}
}

void editColumn(DBE btn)
{
	int i = get(columnListview)
	columnListview_onActivate(columnListview, i)
}

void createNewColumn(DBE btn)
{
	workingColumnLabel = "New Column"
	workingColumnAttrs = null

	ecps_Column_Details_Dialog()

	if columnDetailsOK then
	{
		insert(columnListview, noElems(columnListview), workingColumnLabel)
		set(columnListview, noElems(columnListview)-1, 2, workingColumnAttrs)
		set(columnListview, noElems(columnListview)-1)
		OleAutoObj obj_csUser = null
		OleAutoObj obj_csApiLib = get_csApiLib(obj_csUser)
		Skip candidateAttrs = createString
		getReportableCRAttributeList(obj_csApiLib, obj_csUser, candidateAttrs)
		if csint_status_view_target == IR_STATUS_VIEW then put(candidateAttrs, CSINT_DOORS_OBJECT_ATTR_CM_BASELINE, "_CM_BASELINE")
		string attrLabels = convertCsAttrListToLabelList(workingColumnAttrs, candidateAttrs)
		delete candidateAttrs
		set(columnListview, noElems(columnListview)-1, 1, attrLabels)
	}
}

void deleteColumn(DBE btn)
{
	int i = get(columnListview)
	if i >= 0 then delete(columnListview, i)
}

void moveColumnUp(DBE btn)
{
	rowUp(columnListview, true)
}

void moveColumnDown(DBE btn)
{
	rowDown(columnListview, true)
}

void cancelConfigDetails(DB box)
{
	configDetailsOK = false
	release box
}

void okConfigDetails(DB box)
{
	workingConfigName = get(configNameField)
	workingIncludeApplied = get(configIncludeAppliedToggle)
	delete workingLabelSkip
	delete workingAttrsSkip
	workingLabelSkip = create
	workingAttrsSkip = create
	int i = 0
	for (i=0; i<noElems(columnListview); i++)
	{
		put(workingLabelSkip, i, getColumnValue(columnListview, i, 0))
		put(workingAttrsSkip, i, getColumnValue(columnListview, i, 2))
	}

	configDetailsOK = true
	release box
	
}

// =======================================================================

void ecps_Configuration_Details_Dialog()
{
	string emptyList[]

	string configDetail_DialogTitle = getCsintString("csString_ConfigDetail_DialogTitle")
	configDetailsDB = create(viewConfigDB, configDetail_DialogTitle, styleStandard)
	
	string configDetail_Field_Name = getCsintString("csString_ConfigDetail_Field_Name")
	configNameField = field(configDetailsDB, configDetail_Field_Name, "", 50)

	string configDetail_Toggle_IncludeApplied = getCsintString("csString_ConfigDetail_Toggle_IncludeApplied")
	configIncludeAppliedToggle = toggle(configDetailsDB, configDetail_Toggle_IncludeApplied, false)

	columnListview = listView(configDetailsDB, 0, 518, 5, emptyList)
	set(columnListview, columnListview_onSelect, columnListview_onDeselect, columnListview_onActivate)
	columnListview->"right"->"unattached"
	if csint_status_view_target == IR_STATUS_VIEW then columnListview->"top"->"spaced"->configNameField
	
	string configDetail_Button_EditColumn = getCsintString("csString_ConfigDetail_Button_EditColumn")
	DBE editColumnBtn = button(configDetailsDB, configDetail_Button_EditColumn, editColumn)
	editColumnBtn->"top"->"aligned"->columnListview
	editColumnBtn->"left"->"spaced"->columnListview

	string configDetail_Button_CreateColumn = getCsintString("csString_ConfigDetail_Button_CreateColumn")
	DBE createNewColumnBtn = button(configDetailsDB, configDetail_Button_CreateColumn, createNewColumn)
	createNewColumnBtn->"top"->"spaced"->editColumnBtn
	createNewColumnBtn->"left"->"spaced"->columnListview

	string configDetail_Button_DeleteColumn = getCsintString("csString_ConfigDetail_Button_DeleteColumn")
	DBE deleteColumnBtn = button(configDetailsDB, configDetail_Button_DeleteColumn, deleteColumn)
	deleteColumnBtn->"top"->"spaced"->createNewColumnBtn
	deleteColumnBtn->"left"->"spaced"->columnListview
	
	string configDetail_Button_MoveColumnUp = getCsintString("csString_ConfigDetail_Button_MoveColumnUp")
	DBE moveColumnUpBtn = button(configDetailsDB, configDetail_Button_MoveColumnUp, moveColumnUp)
	moveColumnUpBtn->"top"->"spaced"->columnListview
	
	string configDetail_Button_MoveColumnDown = getCsintString("csString_ConfigDetail_Button_MoveColumnDown")
	DBE moveColumnDownBtn = button(configDetailsDB, configDetail_Button_MoveColumnDown, moveColumnDown)
	moveColumnDownBtn->"top"->"aligned"->moveColumnUpBtn
	moveColumnDownBtn->"left"->"spaced"->moveColumnUpBtn

	close(configDetailsDB, false, cancelConfigDetails)
	string configView_Button_Cancel = getCsintString("csString_ConfigView_Button_Cancel")
	string configView_Button_OK = getCsintString("csString_ConfigView_Button_OK")
	ok(configDetailsDB, configView_Button_Cancel, cancelConfigDetails)
	ok(configDetailsDB, configView_Button_OK, okConfigDetails)

	realize configDetailsDB
	
	string configDetail_Column_ColumnLabel = getCsintString("csString_ConfigDetail_Column_ColumnLabel")
	string configDetail_Column_IncludedAttrs = getCsintString("csString_ConfigDetail_Column_IncludedAttrs")
	insertColumn(columnListview, 0, configDetail_Column_ColumnLabel,   200, iconNone)
	insertColumn(columnListview, 1, configDetail_Column_IncludedAttrs, 314, iconNone)
	insertColumn(columnListview, 2, "",                      1, iconNone)

	set(configNameField, workingConfigName)
	set(configIncludeAppliedToggle, workingIncludeApplied)

	empty(columnListview)

	string cLabel
	for cLabel in workingLabelSkip do
	{
		int j = (int key workingLabelSkip)
		insert(columnListview, j, cLabel, iconNone)
	}

	OleAutoObj obj_csUser = null
	OleAutoObj obj_csApiLib = get_csApiLib(obj_csUser)
	Skip candidateAttrs = createString
	getReportableCRAttributeList(obj_csApiLib, obj_csUser, candidateAttrs)
	if csint_status_view_target == IR_STATUS_VIEW then put(candidateAttrs, CSINT_DOORS_OBJECT_ATTR_CM_BASELINE, "_CM_BASELINE")
	string cAttrs
	for cAttrs in workingAttrsSkip do
	{
		int j = (int key workingAttrsSkip)
		set(columnListview, j, 2, cAttrs)
		string attrLabels = convertCsAttrListToLabelList(cAttrs, candidateAttrs)
		set(columnListview, j, 1, attrLabels)
	}
	delete candidateAttrs

	set(columnListview, 0)
	setFocus(configNameField)
	if csint_status_view_target == IR_STATUS_VIEW then
	{
		inactive configIncludeAppliedToggle
		hide configIncludeAppliedToggle
	}
	block configDetailsDB

	destroy configDetailsDB
}

// =======================================================================
// ECPS Status View Configuration Dialog
// =======================================================================

void displayConfigData(int focusRow)
{
	empty(configListview)
	string cfName = null
	for cfName in configNameSkip do
	{
		int i = (int key configNameSkip)
		if i == activeConfig then
		{
			insert(configListview, i, "Active")
		}
		else
		{
			insert(configListview, i, "")
		}
		set(configListview, i, 1, cfName)
	}
	if focusRow >= 0 then set(configListview, focusRow)
}

void configListview_onSelect(DBE lv, int i)
{
	// Nothing
}

void configListview_onDeselect(DBE lv, int i)
{
	// Nothing
}

void configListview_onActivate(DBE lv, int i)
{
	if i < 0 then return

	find(configNameSkip, i, workingConfigName)
	if !find(configAppliedSkip, i, workingIncludeApplied) then workingIncludeApplied = false
	find(configColumnLabelSkip, i, workingLabelSkip)
	find(configColumnAttrsSkip, i, workingAttrsSkip)

	ecps_Configuration_Details_Dialog()

	if configDetailsOK then
	{
		delete(configNameSkip, i)
		delete(configAppliedSkip, i)
		delete(configColumnLabelSkip, i)
		delete(configColumnAttrsSkip, i)
		put(configNameSkip, i, workingConfigName)
		put(configAppliedSkip, i, workingIncludeApplied)
		put(configColumnLabelSkip, i, workingLabelSkip)
		put(configColumnAttrsSkip, i, workingAttrsSkip)
	}

	displayConfigData(i)
}

void setActiveConfig(DBE btn)
{
	int i = get(configListview)
	if i < 0 then return

	activeConfig = i
	displayConfigData(activeConfig)
}

void editConfig(DBE btn)
{
	int i = get(configListview)
	configListview_onActivate(configListview, i)
}

void createNewConfig(DBE btn)
{
	int i = get(configListview)

	workingConfigName = "New Configuration"
	workingIncludeApplied = false
	workingLabelSkip = create
	workingAttrsSkip = create

	ecps_Configuration_Details_Dialog()

	if configDetailsOK then
	{
		if (noElems(configListview) == 0) then activeConfig = 0
		int nextNewConfigIndex = noElems(configListview)
		put(configNameSkip, nextNewConfigIndex, workingConfigName)
		put(configAppliedSkip, nextNewConfigIndex, workingIncludeApplied)
		put(configColumnLabelSkip, nextNewConfigIndex, workingLabelSkip)
		put(configColumnAttrsSkip, nextNewConfigIndex, workingAttrsSkip)

		displayConfigData(nextNewConfigIndex)
	}
	else
	{
		if i >= 0 then displayConfigData(i)
	}
}

void deleteConfig(DBE btn)
{
	int i = get(configListview)
	if i < 0 then return

	set(configListview, i, false)

	if activeConfig == i then activeConfig = 0
	if noElems(configListview) == 1 then activeConfig = -1

	delete(configNameSkip, i)
	delete(configAppliedSkip, i)
	delete(configColumnLabelSkip, i)
	delete(configColumnAttrsSkip, i)

	int j = 0
	for (j=i+1; j<noElems(configListview); j++)
	{
		find(configNameSkip, j, workingConfigName)
		if !find(configAppliedSkip, j, workingIncludeApplied) then workingIncludeApplied = false
		find(configColumnLabelSkip, j, workingLabelSkip)
		find(configColumnAttrsSkip, j, workingAttrsSkip)
		delete(configNameSkip, j)
		delete(configAppliedSkip, j)
		delete(configColumnLabelSkip, j)
		delete(configColumnAttrsSkip, j)
		put(configNameSkip, j-1, workingConfigName)
		put(configAppliedSkip, j-1, workingIncludeApplied)
		put(configColumnLabelSkip, j-1, workingLabelSkip)
		put(configColumnAttrsSkip, j-1, workingAttrsSkip)
		if activeConfig == j then activeConfig--
	}
	displayConfigData(-1)
}

void cancelViewConfigurations(DB box)
{
	release box
}

void okViewConfigurations(DB box)
{
	Buffer b_queryAttrs = create
	if activeConfig >= 0 then
	{
		find(configColumnAttrsSkip, activeConfig, workingAttrsSkip)
		string cAttrs
		bool firstAttr = true
		for cAttrs in workingAttrsSkip do
		{
			if firstAttr then
			{
				firstAttr = false
			}
			else
			{
				b_queryAttrs += ", "
			}
			b_queryAttrs += cAttrs
		}
	}
	queryAttrs = stringOf(b_queryAttrs)
	delete b_queryAttrs

	saveViewConfigData(csint_status_view_target, queryAttrs, activeConfig, configNameSkip, configAppliedSkip, configColumnLabelSkip, configColumnAttrsSkip, customQueryAttrs, customMappings, customIncludeApplied)

	User uzer = find(doorsname)
	Module m = current Module
	string errMsg = create_ECPS_Status_view(csint_status_view_target, csint_status_view_type, m)

	release box
}

// =======================================================================

void csint_View_Configuration_Dialog(string vTarget, int vType)
{
	string emptyList[]

	string configView_DialogTitle = getCsintString("csString_ConfigView_DialogTitle", csint_status_view_target)
	Module m = current Module
	viewConfigDB = create(m, configView_DialogTitle, styleStandard)

	configListview = listView(viewConfigDB, 0, 418, 6, emptyList)
	set(configListview, configListview_onSelect, configListview_onDeselect, configListview_onActivate)
	configListview->"right"->"unattached"
	
	string configView_Button_SetDefault = getCsintString("csString_ConfigView_Button_SetDefault")
	DBE setActiveConfigBtn = button(viewConfigDB, configView_Button_SetDefault, setActiveConfig)
	setActiveConfigBtn->"top"->"aligned"->configListview
	setActiveConfigBtn->"left"->"spaced"->configListview

	string configView_Button_EditConfig = getCsintString("csString_ConfigView_Button_EditConfig")
	DBE editConfigBtn = button(viewConfigDB, configView_Button_EditConfig, editConfig)
	editConfigBtn->"top"->"spaced"->setActiveConfigBtn
	editConfigBtn->"left"->"spaced"->configListview

	string configView_Button_CreateConfig = getCsintString("csString_ConfigView_Button_CreateConfig")
	DBE createNewConfigBtn = button(viewConfigDB, configView_Button_CreateConfig, createNewConfig)
	createNewConfigBtn->"top"->"spaced"->editConfigBtn
	createNewConfigBtn->"left"->"spaced"->configListview

	string configView_Button_DeleteConfig = getCsintString("csString_ConfigView_Button_DeleteConfig")
	DBE deleteConfigBtn = button(viewConfigDB, configView_Button_DeleteConfig, deleteConfig)
	deleteConfigBtn->"top"->"spaced"->createNewConfigBtn
	deleteConfigBtn->"left"->"spaced"->configListview

	string configView_Button_Cancel = getCsintString("csString_ConfigView_Button_Cancel")
	string configView_Button_OK = getCsintString("csString_ConfigView_Button_OK")
	close(viewConfigDB, false, cancelViewConfigurations)
	ok(viewConfigDB, configView_Button_Cancel, cancelViewConfigurations)
	ok(viewConfigDB, configView_Button_OK, okViewConfigurations)

	realize viewConfigDB
	
	string configView_Column_Status = getCsintString("csString_ConfigView_Column_Status")
	string configView_Column_Config = getCsintString("csString_ConfigView_Column_Config")
	insertColumn(configListview, 0, configView_Column_Status,    80, iconNone)
	insertColumn(configListview, 1, configView_Column_Config, 334, iconNone)

	getViewConfigData(csint_status_view_target, true, queryAttrs, activeConfig, configNameSkip, configAppliedSkip, configColumnLabelSkip, configColumnAttrsSkip, customQueryAttrs, customMappings, customIncludeApplied)

	displayConfigData(activeConfig)

	setSize(viewConfigDB, 592, 203)

	block viewConfigDB

	destroy viewConfigDB

	delete configNameSkip
	delete configAppliedSkip
	delete configColumnLabelSkip
	delete configColumnAttrsSkip
	delete customMappings
}


// =============================================================================================================
// customize attrs only
// =============================================================================================================
DBE CUSTOM_ATTR_LISTVIEW
int CUSTOM_ATTR_LISTVIEW_SELECTED_ROW
DBE CUSTOM_ATTR_LABEL_FIELD

// add OK button
void customAttr_okCB(DB box)
{
	string dAttrPrefix = getDefaultCsintAttrPrefix(csint_status_view_target)

	Skip existDoorsAttrs = getControlledCsintAttrs(customQueryAttrs, customMappings, dAttrPrefix)
	delete customMappings
	customMappings = createString

	Buffer b = create
	string aName
	string aLabel
	bool firstAttr = true
	int i = 0
	aName = getColumnValue(CUSTOM_ATTR_LISTVIEW, i, 0)
	while (!null aName)
	{
		aLabel = getColumnValue(CUSTOM_ATTR_LISTVIEW, i, 1)
		put(customMappings, aName, aLabel)

		if (getCheck(CUSTOM_ATTR_LISTVIEW, i))
		{
			if firstAttr then
			{
				firstAttr = false
			}
			else
			{
				b += ", "
			}
			b += aName

			// create attribute in DOORS if not exist
			//
			string dxlCode = getCsintAttrDxlCode(aName, csint_status_view_target)
			if (!exists attribute aLabel)
			{
				create object type "Text" dxl dxlCode attribute aLabel
			}
			else
			{
				AttrDef ad = find(current Module, aLabel)
				ad = modify(ad, setDXL, dxlCode)
			}

			delete(existDoorsAttrs, aLabel)
		}
		i += 1
		aName = getColumnValue(CUSTOM_ATTR_LISTVIEW, i, 0)
	}

	customQueryAttrs = stringOf(b)
	delete b

	customIncludeApplied = get(customIncludeAppliedToggle)
	saveViewConfigData(csint_status_view_target, queryAttrs, activeConfig, configNameSkip, configAppliedSkip, configColumnLabelSkip, configColumnAttrsSkip, customQueryAttrs, customMappings, customIncludeApplied)

	string attr4remove
	for attr4remove in existDoorsAttrs do
	{
		if (exists attribute attr4remove)
		{
			AttrDef ad = find(current Module, attr4remove)
			if (!null ad) then delete(ad)
		}
	}
	delete existDoorsAttrs
}

bool userEnterDuplicatedCustomAttr(string newLabel)
{
	bool result = false
	int i = 0
	string aName = getColumnValue(CUSTOM_ATTR_LISTVIEW, i, 0)
	while (!null aName)
	{
		if i != CUSTOM_ATTR_LISTVIEW_SELECTED_ROW then
		{
			string aLabel = getColumnValue(CUSTOM_ATTR_LISTVIEW, i, 1)
			if newLabel == aLabel then
			{
				result = true
				break
			}
		}

		i += 1
		aName = getColumnValue(CUSTOM_ATTR_LISTVIEW, i, 0)
	}

	return result
}

bool userEnterExistAttr(string newLabel)
{
	string currentAttrName = getColumnValue(CUSTOM_ATTR_LISTVIEW, CUSTOM_ATTR_LISTVIEW_SELECTED_ROW, 1)

	bool result = false
	if newLabel != currentAttrName then
	{
		AttrDef ad
		for ad in current Module do
		{
			string existAttrName = ad.name
			if newLabel == existAttrName then
			{
				result = true
				break
			}
		}
	}

	return result
}

void applyColumnLabel(DB box)
{
	string newLabel = get(CUSTOM_ATTR_LABEL_FIELD)
	if null newLabel then
	{
		ack getCsintString("csString_EditAttrName_ErrMsg_CanNotBeEmptyString")
	}
	else if userEnterDuplicatedCustomAttr(newLabel) then
	{
		ack getCsintString("csString_EditAttrName_ErrMsg_CustomAttrExist")
	}
	else if userEnterExistAttr(newLabel) then
	{
		ack getCsintString("csString_EditAttrName_ErrMsg_DOORSAttrExist")
	}
	else
	{
		DBE attrListView = CUSTOM_ATTR_LISTVIEW
		int row_index = CUSTOM_ATTR_LISTVIEW_SELECTED_ROW
		set(attrListView, row_index, 1, newLabel)
		release box
		hide(box)
	}
}

void editColumnLabel(DB box)
{
	string attrName
	string attrLabel
	DBE attrListView = CUSTOM_ATTR_LISTVIEW
	int i
	int row_index = -1
	for i in attrListView do
	{
		attrName = getColumnValue(attrListView, i, 0)
		attrLabel = getColumnValue(attrListView, i, 1)
		row_index = i
	}
	if (row_index >= 0) then
	{
		CUSTOM_ATTR_LISTVIEW_SELECTED_ROW = row_index
		string editAttrName_DialogTitle = getCsintString("csString_EditAttrName_DialogTitle")
		string editAttrName_Field_CRAttrName = getCsintString("csString_EditAttrName_Field_CRAttrName")
		string editAttrName_Field_DOORSAttrName = getCsintString("csString_EditAttrName_Field_DOORSAttrName")
		DB editColumnDB = create(box, editAttrName_DialogTitle, styleCentered)
		DBE nameField = field(editColumnDB, editAttrName_Field_CRAttrName, attrName, 25, true)
		DBE labelField = field(editColumnDB, editAttrName_Field_DOORSAttrName, attrLabel, 32)
		string string_Apply = getCsintString("csString_String_Apply")
		apply(editColumnDB, string_Apply, applyColumnLabel)
		realize editColumnDB
		setFocus(labelField)
		CUSTOM_ATTR_LABEL_FIELD = labelField
		block editColumnDB
	}
	else
	{
		ack getCsintString("csString_EditAttrName_ErrMsg_MustSelectOneAttr")
	}
}

void get_custom_attrs(string vTarget, DBE attrListView)
{
	string dAttrPrefix = getDefaultCsintAttrPrefix(vTarget)

	OleAutoObj obj_csUser = null
	OleAutoObj obj_csApiLib = get_csApiLib(obj_csUser)

	Skip availableAttrList = createString
	getReportableCRAttributeList(obj_csApiLib, obj_csUser, availableAttrList)
	if vTarget == IR_ATTRS_VIEW then put(availableAttrList, CSINT_DOORS_OBJECT_ATTR_CM_BASELINE, "_CM_BASELINE")

	getViewConfigData(vTarget, true, queryAttrs, activeConfig, configNameSkip, configAppliedSkip, configColumnLabelSkip, configColumnAttrsSkip, customQueryAttrs, customMappings, customIncludeApplied)
	Skip csAttrs = createString
	convertCommaSeperatedAttrsStringToSkip(customQueryAttrs, csAttrs)

	string attrName
	string attrLabel
	string attrDoorsName
	int listViewIndex = 0
	for attrLabel in availableAttrList do
	{
		attrName = (string key availableAttrList)
		if !find(customMappings, attrName, attrDoorsName) then attrDoorsName = dAttrPrefix attrName
		insert(attrListView, listViewIndex, attrName, iconNone)
		set(attrListView, listViewIndex, 1, attrDoorsName)
		if (find(csAttrs, attrName))
		{
			setCheck(attrListView, listViewIndex, true)
		}
		listViewIndex += 1
	}

	delete availableAttrList
	delete csAttrs
}

// ===========================================================
// This function is for custom attrs
// ===========================================================
void customAttrDialog(string vTarget, int vType)
{
	string dbTitle = RCP
	if vTarget == IR_ATTRS_VIEW then dbTitle = ECR

	string customAttrs_DialogTitle = getCsintString("csString_CustomAttrs_DialogTitle", dbTitle)
	DB customDB = create(current Module, customAttrs_DialogTitle, styleCentered)
	string emptyList[]

	// this toggle control only used by RCR
	string customAttrs_Toggle_IncludeApplied = getCsintString("csString_CustomAttrs_Toggle_IncludeApplied")
	customIncludeAppliedToggle = toggle(customDB, customAttrs_Toggle_IncludeApplied, false)

	string customAttrs_Button_EditAttrName = getCsintString("csString_CustomAttrs_Button_EditAttrName")
	string string_Apply = getCsintString("csString_String_Apply")
	apply(customDB, customAttrs_Button_EditAttrName, editColumnLabel)
	ok(customDB, string_Apply, customAttr_okCB)

	string customAttrs_Frame_AttrSetting = getCsintString("csString_CustomAttrs_Frame_AttrSetting", dbTitle)
	DBE attrSettingFrame = frame(customDB, customAttrs_Frame_AttrSetting, 400, 240)
	if csint_status_view_target == IR_ATTRS_VIEW then attrSettingFrame->"top"->"form"
	attrSettingFrame->"bottom"->"form"

	DBE attrListView = listView(customDB, listViewOptionCheckboxes, 100, 3, emptyList)
	attrListView->"top"->"inside"->attrSettingFrame
	attrListView->"left"->"inside"->attrSettingFrame
	attrListView->"right"->"inside"->attrSettingFrame
	attrListView->"bottom"->"inside"->attrSettingFrame

	realize customDB

	string customAttrs_Column_Attr = getCsintString("csString_CustomAttrs_Column_Attr", dbTitle)
	string customAttrs_Column_DOORSAttrName = getCsintString("csString_CustomAttrs_Column_DOORSAttrName")
	insertColumn(attrListView, 0, customAttrs_Column_Attr, 200,iconNone)
	insertColumn(attrListView, 1, customAttrs_Column_DOORSAttrName, 250,iconNone)

	//get_CR_AttributeList(attrListView)
	get_custom_attrs(vTarget, attrListView)	// read config data and populate the list view
	
	realize customDB
	setCenteredSize(customDB, 540, 320)
	set(customIncludeAppliedToggle, customIncludeApplied)

	CUSTOM_ATTR_LISTVIEW = attrListView

	// hide and disable the following toggle if it is for IR
	if csint_status_view_target == IR_ATTRS_VIEW then
	{
		inactive customIncludeAppliedToggle
		hide customIncludeAppliedToggle
	}

	block customDB
	destroy customDB

	// clean up
	delete configNameSkip
	delete configAppliedSkip
	delete configColumnLabelSkip
	delete configColumnAttrsSkip
	delete customMappings
}

void ecps_View_Configuration_Dialog(string vTarget, int vType)
{
	csint_status_view_target = vTarget
	csint_status_view_type = vType

	if ((vTarget == IR_ATTRS_VIEW) || (vTarget == ECPS_ATTRS_VIEW)) then
	{
		customAttrDialog(vTarget, vType)
	}
	else if ((vTarget == IR_STATUS_VIEW) || (vTarget == ECPS_STATUS_VIEW)) then
	{
		csint_View_Configuration_Dialog(vTarget, vType)
	}
	else
	{
		// something wrong
	}
}
