/**********************************************************************
*
*	DXL:		syncAllModules56.dxl
*	Instance:		1
*	Description:	Change/DOORS Integration
*			Synchronize the "Change Request" and "CR Status" attributes
*			in all the requirement objects in current module from ChangeSynergy
*	%created_by:	usmave %
*	%date_created:	Monday, October 26, 2009 8:15:28 AM %
*       %modified by:   Nancy Coelho %
*	%date_created:	Monday, October 26, 2009 8:15:28 AM %
*   Copyright (c) 2001 Telelogic AB
**********************************************************************/






// =====================================================================================

Skip dbModuleFullNames  = create
int numFolders = 0
int numModules = 0
int progress = 0
bool showProgressBar =(!isBatch())

bool mSectionPushToCs(Module mdl, OleAutoObj obj_csApiLib, obj_csUser, string queryString)
{
	bool continueSyncProcess = true
        Module m = mdl
	csintDebug_("Query CS for associated req objects with query string = " queryString)
	OleAutoObj obj_csQueryData = do_csApiLib_QueryStringData(obj_csApiLib, obj_csUser, "", queryString, "name|cvid|doors_modify_time")
	csintDebug_("Query return.")
	if (!null obj_csQueryData)
	{
		int numResults = get_int_from_OleAutoObj(obj_csQueryData, "DataSize")
		csintDebug_("The number of associated req objects return by query is " numResults ".")
		int j
		for(j=0;j<numResults;j++)
		{
			OleAutoObj obj_csObjectVector = get_indexed_OleAutoObj_from_OleAutoObj(obj_csQueryData, "DataObject", j)
			if (!null obj_csObjectVector)
			{
				string cvid
				int modtime
				string reqName
				int i
				for (i=0; i<3; i++)
				{
					OleAutoObj obj_csObjectData = get_indexed_OleAutoObj_from_OleAutoObj(obj_csObjectVector, "DataObject", i)
					string oName = get_string_from_OleAutoObj(obj_csObjectData, "Name")
					
					if (oName == "cvid")
					{
						cvid = get_string_from_OleAutoObj(obj_csObjectData, "Value")
					}
					else if (oName == "doors_modify_time")
					{
						modtime = intOf(get_string_from_OleAutoObj(obj_csObjectData, "Value"))
					}
					else if (oName == "name")
					{
						reqName = get_string_from_OleAutoObj(obj_csObjectData, "Value")
					}
					else
					{
						continue
					}
				}

				string oID = getLastElement(reqName, '_')
				csintDebug_("Data[" j "] cvid=" cvid " doors_modify_time=" modtime " objAbsNo=" oID "")
				Object obj = findThisObject(mdl, oID)
				push_Obj_Data_to_CS(obj_csApiLib, obj_csUser, obj, cvid, modtime, m)		
			}
			else
			{
				csintDebug_("Data[" j "] is null")
			}
		}
		cleanupQueryData(obj_csQueryData)
	}

	return continueSyncProcess
}

// =====================================================================================
bool mSectionUpdateDOORS(Module mdl, OleAutoObj obj_csApiLib, obj_csUser, string secondQueryString, queryString, attrList)
{
	bool continueSyncProcess = true
	int iNumECRs = 0
	int iNumReqs = 0

	csintDebug_("Query CS for associated IRs with query string = " secondQueryString)
	OleAutoObj csQueryDataCR = do_csApiLib_QueryStringData(obj_csApiLib, obj_csUser, "", secondQueryString, attrList)
	csintDebug_("Query return.")
	if (!null csQueryDataCR)
	{
		iNumECRs = get_int_from_OleAutoObj(csQueryDataCR, "DataSize")
		csintDebug_("The number of associated ECRs return by query is " iNumECRs ".")
	}

	csintDebug_("Query CS for associated req objects using doors_data_report_req_cr report with query string = " queryString)
	OleAutoObj csQueryDataReq2CR = do_csApiLib_QueryStringData(obj_csApiLib, obj_csUser, "doors_data_report_req_cr", queryString,"name|cvid")
	csintDebug_("Query return.")
	if (!null csQueryDataReq2CR)
	{
		iNumReqs = get_int_from_OleAutoObj(csQueryDataReq2CR, "DataSize")
		csintDebug_("The number of associated req objects return by query is " iNumReqs ".")
	}

	if ((iNumECRs>0)&&(iNumReqs>0)) then
	{
		Skip dataSkip = createString
		indexCRInQueryData(csQueryDataCR, dataSkip)
		Buffer cacheBuffer = create

		int iReq
		for (iReq=0; iReq<iNumReqs; iReq++)
		{
			OleAutoObj reqObj = get_indexed_OleAutoObj_from_OleAutoObj(csQueryDataReq2CR, "DataObject", iReq)
			if (!null reqObj)
			{
				string errMsg = ""
				string doorsID = getRecordField(reqObj, "name", errMsg)
				string objID = getLastElement(doorsID, '_')
				csintDebug_("Data[" iReq "] for object " objID)
				Object obj = findThisObject(mdl, objID)
				if (!null obj)
				{
					update_IR_StatusCache(mdl, obj_csApiLib, obj_csUser, obj, dataSkip, reqObj, cacheBuffer)
				}
				else
				{
					csintDebug_("\tObject(" objID ") can not be found in DOORS! Skip update this object in DOORS.")
				}
			}
			else
			{
				csintDebug_("Data[" iReq "] is null")
			}
		}
		delete dataSkip
		delete cacheBuffer
	}

	if (!null csQueryDataCR) then cleanupQueryData(csQueryDataCR)
	if (!null csQueryDataReq2CR) then cleanupQueryData(csQueryDataReq2CR)

	return continueSyncProcess
}

// =====================================================================================
void module_ir_sync(string viewName, Module mode)
{
	csintDebug_("module_ir_sync(" viewName ") start...")
	int startTime = intOf today()

	Object cur_obj = current Object
	Module mdl = mode // current Module
	if (null mdl)
	{
		string addIR_ErrMsg_NoSpecifiedIR = getCsintString("csString_AddIR_ErrMsg_NoSpecifiedIR")
		abortMsg(addIR_ErrMsg_NoSpecifiedIR)
	}
	if (!canRead mdl)
	{
		string csint_ErrMsg_NoReadAccessToModule = getCsintString("csString_Csint_ErrMsg_NoReadAccessToModule")
		abortMsg(csint_ErrMsg_NoReadAccessToModule)
	}
	
	Item modItem = item fullName mdl
	string mid = uniqueID modItem

	string attrList = ECPS_DEFAULT_CACHE_QUERY_ATTRS
	if IR_USE_STATUS_VIEW then getAttrListFromViewConfig(IR_STATUS_VIEW, attrList)
	if IR_USE_ATTRS_VIEW then getAttrListFromViewConfig(IR_ATTRS_VIEW, attrList)

	OleAutoObj obj_csUser = null
	OleAutoObj obj_csApiLib = get_csApiLib(obj_csUser)
	Buffer b = create
	int xNumber
	bool continueSyncProcess = true

	if continueSyncProcess then
	{
		csintDebug_("====================================================")
		csintDebug_("Phase.1 Update requirement objects in Change first")
		csintDebug_("====================================================")
		for(xNumber=0; xNumber<10; xNumber++)
		{
			csintDebug_("push to cs -----------------------------> " xNumber "...")
			string req_name = "req" mid "_" xNumber "*"

			setempty b
			b += "name match '"
			b += req_name
			b += "'"
			string queryString = stringOf b

			continueSyncProcess = mSectionPushToCs(mdl, obj_csApiLib, obj_csUser, queryString)
			if !continueSyncProcess then break
		}
		csintDebug_("end of push to cs.")

		csintDebug_("====================================================")
		csintDebug_("Phase.2 Update IR info in DOORS")
		csintDebug_("====================================================")
		for(xNumber=0; xNumber<10; xNumber++)
		{
			csintDebug_("update doors -----------------------------> " xNumber "...")
			string req_name = "req" mid "_" xNumber "*"

			setempty b
			b += "name match '"
			b += req_name
			b += "'"
			string queryString = stringOf b

			setempty b
			b += "has_associated_req(name match '"
			b += req_name
			b += "')"
			string secondQueryString = stringOf b

			continueSyncProcess = mSectionUpdateDOORS(mdl, obj_csApiLib, obj_csUser, secondQueryString, queryString, attrList)
			if !continueSyncProcess then break
		}
		csintDebug_("end of update doors.")
	}

	delete b

	refresh mdl

	int endTime = intOf today()
	csintDebug_("module_ir_sync() end and took " (endTime-startTime) " seconds")
}



void syncOneModule(Item i)
{
	string moduleFullName = fullName(i)
	string mid = uniqueID(i)
	
	Module m = read(moduleFullName, false)
		if (null m)
		{
		    print("syncOneModule : Nancy " moduleFullName "\n" )
			return;
		}
		else
		{
			if(riIsEnabled(mid))
			{
				module_ir_sync(IR_ATTRS_VIEW, m)
				// print("module_ir_sync : Nancy " moduleFullName "\n" )
			}
			if(rcmIsEnabled(mid))
			{
				//print("updateCPstatusAndRefreshStatusAttribute : Nancy " moduleFullName "\n" )
				string viewName = ECPS_ATTRS_VIEW
				current= m
				updateCPstatusAndRefreshStatusAttribute(viewName, null)
			}
			close m
		}
	
	
}

void getOpenModuleFullName(Skip &dbModuleFullNames, Folder f, int &numFolders, int &numModules)
{
	numFolders++
	csintDebug_("Checking folder/project: " fullName(f))

	Item i
	current = f
	for i in all f do
	{
		if isDeleted(i) then continue
		string itemType = type(i)
		if ((itemType == "Project") || (itemType == "Folder"))
		{
			getOpenModuleFullName(dbModuleFullNames, folder(i), numFolders, numModules)
		}
		else if (itemType == "Formal")
		{
			numModules++
			Module m = module(i)
			if (!null m) then	// means the module is open
			{			
				string modPath = fullName(m)
				put(dbModuleFullNames, modPath, modPath)
			}
		}
	}
}


void processFolder(Folder f)
{
	Item i
	current = f
	for i in all f do
	{
		if isDeleted(i) then continue
		string itemType = type(i)
		if ((itemType == "Project") || (itemType == "Folder"))
		{
			processFolder(folder(i))
		}
		else if (itemType == "Formal")
		{
		  string moduleFullName = fullName(i)
		  string mid = uniqueID(i)
		 if(csintIsCsintON(mid))
		 {
		  
			if (progressCancelled()) then
			{
				csintDebug_("User cancel the sync process!!")
				break
			}
			else
			{
				 progress++
				if showProgressBar then
				{
					progressStep(progress)
				}
				syncOneModule(i)
			}
		}

		}
	}
	
}

void loopOverAllModules()
{
	
	int startTime = intOf today()
	Folder f = folder "/"
	DB myBox = null
	bool allowCancel = true
	Item i
	// num of modules
	getOpenModuleFullName(dbModuleFullNames, f, numFolders, numModules)
	int isteps=numModules
	
	if showProgressBar then
	{
		string string_Progress = getCsintString("csString_String_Progress")
		myBox = create(string_Progress)
		realize myBox
		hide myBox
		string string_SyncAll_Modules= getCsintString("csString_SyncAll_Modules")
		progressStart(myBox, string_SyncAll_Modules, "", numModules)
		
		string string_Sync_ALL = getCsintString("csString_String_Sync_ALL")
		progressMessage(string_Sync_ALL)
	}
	for i in all f do
	{
	      
		string itemType = type(i)
		if (itemType == "Project")
		{ //
			current = project(i)
		}
		else if (itemType == "Folder")
		{ //
			current = folder(i)
		}
		
		
		processFolder(folder(i))
	}
	int endTime = intOf today()
        if showProgressBar then
	{
		progressStop()
		hide myBox
		destroy myBox
	}
}

loopOverAllModules()
